<?php

namespace Drupal\mailchimp_marketing;

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ConnectException;
use MailchimpMarketing\ApiClient;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Provides Mailchimp services.
 */
class Mailchimp implements MailchimpInterface {

  use StringTranslationTrait;

  /**
   * Config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The email validator.
   *
   * @var \Drupal\Component\Utility\EmailValidatorInterface
   */
  protected $emailValidator;

  /**
   * The Logger channel service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $loggerChannel;

  /**
   * {@inheritDoc}
   */
  public function __construct(LoggerChannelFactoryInterface $logger, ConfigFactoryInterface $config_factory, EmailValidatorInterface $validator) {
    $this->loggerChannel = $logger->get('mailchimp_marketing');
    $this->config = $config_factory->get('mailchimp_marketing.settings');
    $this->emailValidator = $validator;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container): Mailchimp {
    /** @var \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger */
    $logger = $container->get('logger.factory');
    /** @var \Drupal\Core\Config\ConfigFactoryInterface $config_factory */
    $config_factory = $container->get('config.factory');
    /** @var \Drupal\Component\Utility\EmailValidatorInterface $validator */
    $validator = $container->get('email.validator');

    return new static(
      $logger,
      $config_factory,
      $validator
    );
  }

  /**
   * {@inheritDoc}
   */
  public function getConnection() {
    $api_key = $this->config->get('mailchimp_api_key');
    $server_prefix = $this->config->get('mailchimp_server_prefix');

    // Test the connection.
    $mailchimp = new ApiClient();
    $mailchimp->setConfig([
      'apiKey' => $api_key,
      'server' => $server_prefix,
    ]);

    return $mailchimp;
  }

  /**
   * {@inheritDoc}
   */
  public function pingSuccess() {
    try {
      $mailchimp = $this->getConnection();
      $response = $mailchimp->ping->get();
      return ($response->health_status === "Everything's Chimpy!");
    }
    catch (ClientException $e) {
      return FALSE;
    }
    catch (ConnectException $e) {
      return FALSE;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function getDefaultList() {
    return $this->config->get('mailchimp_default_list');
  }

  /**
   * {@inheritDoc}
   */
  public function getMember($email, $listId = NULL) {
    $mailchimp = $this->getConnection();

    if (!$this->emailValidator->isValid($email)) {
      $this->loggerChannel->notice("Invalid email address.");
      return FALSE;
    }

    if (!isset($listId)) {
      $listId = $this->config->get('mailchimp_default_list');
    }

    if (!isset($listId)) {
      $this->loggerChannel->warning("Mailchimp list is undefined.");
      return FALSE;
    }

    try {
      $memberId = md5(strtolower($email));
      $response = $mailchimp->lists->getListMember($listId, $memberId);
    }
    catch (\Exception $exception) {
      $this->loggerChannel->warning($exception->getMessage());
      return FALSE;
    }

    return $response;
  }

  /**
   * {@inheritDoc}
   */
  public function getMemberByUniqueId(string $uniqueId, $listId = NULL) {
    $mailchimp = $this->getConnection();

    if (empty($uniqueId)) {
      $this->loggerChannel->notice("Invalid unique email id.");
      return FALSE;
    }

    if (!isset($listId)) {
      $listId = $this->config->get('mailchimp_default_list');
    }

    if (!isset($listId)) {
      $this->loggerChannel->warning("Mailchimp list is undefined.");
      return FALSE;
    }

    try {
      $response = $mailchimp->lists->getListMembersInfo($listId, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, $uniqueId);
      if ($response->total_items > 0) {
        $response = $response->members[0];
      }
      else {
        return FALSE;
      }
    }
    catch (\Exception $exception) {
      $this->loggerChannel->error($exception->getMessage());
      return FALSE;
    }

    return $response;
  }

  /**
   * {@inheritDoc}
   */
  public function addMember(string $email, string $listId = NULL, array $options = [], string $memberStatus = 'subscribed') {
    if (!isset($listId)) {
      $listId = $this->config->get('mailchimp_default_list');
    }

    if (!isset($listId)) {
      $this->loggerChannel->warning("Mailchimp list is undefined.");
      return FALSE;
    }

    $member = $this->getMember($email, $listId);

    if (!$member) {
      // Add new member.
      $request = [
        "email_address" => $email,
        "status" => $memberStatus,
      ];

      if (isset($options)) {
        $request = array_merge($request, $options);
      }

      try {
        $mailchimp = $this->getConnection();

        // To successfully add tags to new user, custom keys should be removed.
        if (isset($request['tags']) && is_array($request['tags'])) {
          $mailchimp_tags_no_ids = [];
          foreach ($request['tags'] as $mailchimp_tag) {
            $mailchimp_tags_no_ids[] = $mailchimp_tag;
          }
          $request['tags'] = $mailchimp_tags_no_ids;
        }

        $member = $mailchimp->lists->addListMember($listId, $request);
      }
      catch (\Exception $exception) {
        $this->loggerChannel->error($exception->getMessage());
        return FALSE;
      }
    }
    else {
      // Check user's status.
      if ($member->status === 'unsubscribed') {
        $this->loggerChannel->error($this->t('User has unsubscribed from this mail list and cannot be re-subscribed.'));
        return FALSE;
      }

      if ($member->status === 'pending') {
        $this->loggerChannel->error($this->t('User has subscribed to this list, but has not confirmed their subscription.'));
        return FALSE;
      }

      // Update existing subscriber.
      try {
        $mailchimp = $this->getConnection();
        if (isset($options) && isset($options['tags']) && count($options['tags']) > 0) {
          $requestTags = [];
          $mailchimpTags = $mailchimp->lists->getListMemberTags($listId, $member->id, NULL, NULL, 200);
          $tags = $options['tags'];

          // Set members tags from mailchimp to inactive.
          // @todo remove tags if options is empty.
          foreach ($mailchimpTags->tags as $tag) {
            if (!in_array($tag->name, $tags)) {
              $requestTags[] = [
                "name" => $tag->name,
                "status" => 'inactive',
              ];
            }
          }

          // Set new tags to active.
          foreach ($tags as $tag) {
            $requestTags[] = [
              "name" => $tag,
              "status" => "active",
            ];
          }

          $request = [
            "tags" => $requestTags,
          ];

          $mailchimp->lists->updateListMemberTags(
            $listId,
            $member->id,
            $request
          );
        }

        unset($options['tags']);
        if (!empty($options)) {
          $member = $mailchimp->lists->updateListMember(
            $listId,
            $member->id,
            $options
          );
        }
      }
      catch (\Exception $exception) {
        $this->loggerChannel->error($exception->getMessage());
        return FALSE;
      }
    }

    return isset($member) ? $member : FALSE;
  }

}
